/**
 * Quad Sim v0.21
 * Copyright (C) 2005  Giliam de Carpentier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * You can contact me at giliam@decarpentier.nl for questions concerning this license.
 */
// included by "common.h"
#ifndef QUAD_H
#define QUAD_H


// ----------------------------------------------------------------------------
// CLASS: AXLE declaration
// DESCR: superclass that controls an axle coupled to a parent MB
// ----------------------------------------------------------------------------
class AXLE : public MB {
	float relAlpha, oldParentOmega;
public:
	float relPhi, relOmega;
	MB *pmb;

	AXLE(MB *pmb, float theta, float relPhi, float relOmega);
	void addFriction(float max);
	void pre();
	void advance();
	void post();
};

// ----------------------------------------------------------------------------
// CLASS: WHEEL declaration
// DESCR: defines a WHEEL based on MPs coupled to an AXIS
//        can add MPs forces to AXIS and render the WHEEL
// ----------------------------------------------------------------------------
class WHEEL {
public:
	MP **ppmp;
	AXLE *paxle; 
	SPRINGBP **ppspring;
	int n; float r, w, cFric, cFric0, cFricProf, gripFact; V vC, vProf;

	WHEEL(AXLE *paxle, int n, const V &vC, float r, float w,
		  float mpm, float cS, float cD, float cFric, float cFric0, const V &vProf);
	~WHEEL();
	void pre();
	void advance();
	void post();
	void render();
};

// ----------------------------------------------------------------------------
// CLASS: BSS (Back Sub-System) declaration
// DESCR: defines the back swing arm, axle and wheels
// ----------------------------------------------------------------------------
class BSS {
public:
	class QUAD *pquad;
	MB *pbearing;
	AXLE *paxle;
	SPRINGBB *parm, *pshock;
	WHEEL *pwheelL, *pwheelR;

	BSS(QUAD *pquad);
	~BSS();
	void pre();
	void advance();
	void post();
	void render();
};


// ----------------------------------------------------------------------------
// CLASS: FSS (Front Sub-System) declaration
// DESCR: defines the back swing arm, axle and wheels
// ----------------------------------------------------------------------------
class FSS {
public:
	class QUAD *pquad;
	MB *pbearing;
	AXLE *paxle;
	SPRINGBB *parm, *pshock;
	WHEEL *pwheel;

	float casterDist; float side;
	RM rmCaster, rmCasterDelta;
	V vSteerN;
	

	FSS(QUAD *pquad, bool left);
	~FSS();
	void pre();
	void advance();
	void post();
	void render();
};


// ----------------------------------------------------------------------------
// CLASS: QUAD declaration
// DESCR: defines a quad
// ----------------------------------------------------------------------------
class QUAD {
public:
	MB *pmb;
	BSS *pbss;
	FSS *pfssL, *pfssR;

	QUAD(MB *pmb);
	void pre();
	void advance();
	void post();
	void render();
};



#endif


